from datetime import datetime
from flask import render_template, flash, redirect, url_for, request
from flask_login import login_required
from app import db
from app.admin.forms import EditUserForm, AdminResetPasswordForm, NewUserForm
# from app.main.forms import EditUserForm, AdminResetPasswordForm, NewUserForm
from app.auth.permissions import admin_permission
from app.models import User
from app.admin import bp

@bp.route('/admin', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def admin():
    return render_template('admin/admin.html', title='Admin Panel')


@bp.route('/admin/user_management', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def user_management():
    users = User.query.all()

    return render_template('admin/user_management.html', title='Admin Panel - User Management', users=[u for u in users])


@bp.route('/admin/user_management/edit_user/<username>', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def edit_user(username):
    form_user = User.query.filter_by(username=username).first_or_404()
    form = EditUserForm()

    if form.validate_on_submit():
        form_user.username = form.username.data
        form_user.email = form.email.data
        form_user.first_name = form.first_name.data
        form_user.last_name = form.last_name.data
        form_user.role = form.role.data

        if form.active.data:
            form_user.set_active()
        else:
            form_user.set_inactive()

        db.session.commit()
        flash("{0}'s profile updated".format(form_user.username))
        return render_template('index.html', title="Admin Panel - Edit {0}'s profile".format(form_user.username), form=form)

    elif request.method == 'GET':
        form.username.data = form_user.username
        form.email.data = form_user.email
        form.first_name.data = form_user.first_name
        form.last_name.data = form_user.last_name
        form.role.data = form_user.role
        form.active.data = form_user.active

    return render_template('index.html', title="Admin Panel - Edit {0}'s profile".format(form_user.username), form=form)


@bp.route('/admin/user_management/new_user', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def new_user():
    form = NewUserForm()
    if form.validate_on_submit():
        user = User()
        user.username = form.username.data
        user.email = form.email.data
        user.email_confirmed_at = datetime.utcnow()
        user.set_password(form.password2.data)
        user.first_name = form.first_name.data
        user.last_name = form.last_name.data
        user.role = form.role.data

        db.session.add(user)
        db.session.commit()
        flash("User {} created".format(form.username.data))

    elif request.method == 'GET':
        return render_template('index.html', title='New user', form=form)

    return render_template('index.html', title='Admin Panel - New user', form=form)


@bp.route('/admin/user_management/disable_user/<username>', methods=['POST'])
@login_required
def disable_user(username):
    user = User.query.filter_by(username=username).first_or_404()
    user.is_disabled = True
    user.role = "disabled"
    db.session.commit()
    return redirect(url_for('main.link_properties'))


@bp.route('/admin/user_management/reset_password/<username>', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def admin_reset_password(username):
    user = User.query.filter_by(username=username).first()
    form = AdminResetPasswordForm()
    if form.validate_on_submit():
        user.set_password(form.password.data)
        db.session.commit()
        flash("User's password has been reset.")
        return redirect(url_for('main.user', username=user.username))
    return render_template('auth/reset_password.html', form=form, username=user.username)


@bp.route('/admin/landlord_management', methods=['GET', 'POST'])
@login_required
@admin_permission.require()
def landlord_management():
    users = User.query.filter_by(role="landlord").all()
    return render_template('admin/landlord_management.html', title='Admin Panel - Landlord Management', users=[u for u in users])